package control

import (
	rawclient "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc/client"
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-cli/internal/key"
	commonCmd "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/internal/common"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/control"
	"github.com/spf13/cobra"
)

const (
	healthcheckIRFlag = "ir"
)

var healthCheckCmd = &cobra.Command{
	Use:   "healthcheck",
	Short: "Health check for FrostFS storage nodes",
	Long:  "Health check for FrostFS storage nodes.",
	Run:   healthCheck,
}

func initControlHealthCheckCmd() {
	initControlFlags(healthCheckCmd)

	flags := healthCheckCmd.Flags()
	flags.Bool(healthcheckIRFlag, false, "Communicate with IR node")
	_ = flags.MarkDeprecated(healthcheckIRFlag, "for health check of inner ring nodes, use the 'control ir healthcheck' command instead.")
}

func healthCheck(cmd *cobra.Command, args []string) {
	if isIR, _ := cmd.Flags().GetBool(healthcheckIRFlag); isIR {
		irHealthCheck(cmd, args)
		return
	}

	pk := key.Get(cmd)
	cli := getClient(cmd, pk)

	req := new(control.HealthCheckRequest)
	req.SetBody(new(control.HealthCheckRequest_Body))

	signRequest(cmd, pk, req)

	var resp *control.HealthCheckResponse
	var err error
	err = cli.ExecRaw(func(client *rawclient.Client) error {
		resp, err = control.HealthCheck(client, req)
		return err
	})
	commonCmd.ExitOnErr(cmd, "rpc error: %w", err)

	verifyResponse(cmd, resp.GetSignature(), resp.GetBody())

	cmd.Printf("Network status: %s\n", resp.GetBody().GetNetmapStatus())
	cmd.Printf("Health status: %s\n", resp.GetBody().GetHealthStatus())
}
