package control

import (
	rawclient "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc/client"
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-cli/internal/key"
	commonCmd "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/internal/common"
	ircontrol "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/control/ir"
	ircontrolsrv "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/control/ir/server"
	"github.com/spf13/cobra"
)

var tickEpochCmd = &cobra.Command{
	Use:   "tick-epoch",
	Short: "Forces a new epoch",
	Long:  "Forces a new epoch via a notary request. It should be executed on other IR nodes as well.",
	Run:   tickEpoch,
}

func initControlIRTickEpochCmd() {
	initControlFlags(tickEpochCmd)
}

func tickEpoch(cmd *cobra.Command, _ []string) {
	pk := key.Get(cmd)
	c := getClient(cmd, pk)

	req := new(ircontrol.TickEpochRequest)
	req.SetBody(new(ircontrol.TickEpochRequest_Body))

	err := ircontrolsrv.SignMessage(pk, req)
	commonCmd.ExitOnErr(cmd, "could not sign request: %w", err)

	var resp *ircontrol.TickEpochResponse
	err = c.ExecRaw(func(client *rawclient.Client) error {
		resp, err = ircontrol.TickEpoch(client, req)
		return err
	})
	commonCmd.ExitOnErr(cmd, "rpc error: %w", err)

	verifyResponse(cmd, resp.GetSignature(), resp.GetBody())

	cmd.Println("Epoch tick requested")
}
