package writecache

import (
	common "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-lens/internal"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/writecache"
	"git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object"
	"github.com/spf13/cobra"
)

var inspectCMD = &cobra.Command{
	Use:   "inspect",
	Short: "Object inspection",
	Long:  `Inspect specific object in a write-cache.`,
	Run:   inspectFunc,
}

func init() {
	common.AddAddressFlag(inspectCMD, &vAddress)
	common.AddComponentPathFlag(inspectCMD, &vPath)
	common.AddOutputFileFlag(inspectCMD, &vOut)
}

func inspectFunc(cmd *cobra.Command, _ []string) {
	db := openWC(cmd)
	defer db.Close()

	data, err := writecache.Get(db, []byte(vAddress))
	common.ExitOnErr(cmd, common.Errf("could not fetch object: %w", err))

	var o object.Object
	common.ExitOnErr(cmd, common.Errf("could not unmarshal object: %w", o.Unmarshal(data)))

	common.PrintObjectHeader(cmd, o)
	common.WriteObjectToFile(cmd, vOut, data)
}
