package netmap

import (
	v2netmap "git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/netmap"
	"git.frostfs.info/TrueCloudLab/frostfs-node/internal/logs"
	netmapclient "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/client/netmap"
	"github.com/nspcc-dev/neo-go/pkg/crypto/keys"
	"go.uber.org/zap"
)

func (np *Processor) processNetmapCleanupTick(ev netmapCleanupTick) {
	if !np.alphabetState.IsAlphabet() {
		np.log.Info(logs.NetmapNonAlphabetModeIgnoreNewNetmapCleanupTick)

		return
	}

	err := np.netmapSnapshot.forEachRemoveCandidate(ev.epoch, func(s string) error {
		key, err := keys.NewPublicKeyFromString(s)
		if err != nil {
			np.log.Warn(logs.NetmapCantDecodePublicKeyOfNetmapNode,
				zap.String("key", s))

			return nil
		}

		np.log.Info(logs.NetmapVoteToRemoveNodeFromNetmap, zap.String("key", s))

		// In notary environments we call UpdateStateIR method instead of UpdateState.
		// It differs from UpdateState only by name, so we can do this in the same form.
		// See https://github.com/nspcc-dev/frostfs-contract/issues/225
		const methodUpdateStateNotary = "updateStateIR"

		if np.notaryDisabled {
			prm := netmapclient.UpdatePeerPrm{}

			prm.SetKey(key.Bytes())
			prm.SetHash(ev.TxHash())

			err = np.netmapClient.UpdatePeerState(prm)
		} else {
			err = np.netmapClient.MorphNotaryInvoke(
				np.netmapClient.ContractAddress(),
				0,
				uint32(ev.epoch),
				nil,
				methodUpdateStateNotary,
				int64(v2netmap.Offline), key.Bytes(),
			)
		}
		if err != nil {
			np.log.Error(logs.NetmapCantInvokeNetmapUpdateState, zap.Error(err))
		}

		return nil
	})
	if err != nil {
		np.log.Warn(logs.NetmapCantIterateOnNetmapCleanerCache,
			zap.String("error", err.Error()))
	}
}
