package metrics

import "github.com/prometheus/client_golang/prometheus"

const innerRingSubsystem = "ir"

// InnerRingServiceMetrics contains metrics collected by inner ring.
type InnerRingServiceMetrics struct {
	epoch  metric[prometheus.Gauge]
	health metric[prometheus.Gauge]
}

// NewInnerRingMetrics returns new instance of metrics collectors for inner ring.
func NewInnerRingMetrics() *InnerRingServiceMetrics {
	var (
		epoch = newGauge(prometheus.GaugeOpts{
			Namespace: namespace,
			Subsystem: innerRingSubsystem,
			Name:      "epoch",
			Help:      "Current epoch as seen by inner-ring node.",
		})
		health = newGauge(prometheus.GaugeOpts{
			Namespace: namespace,
			Subsystem: innerRingSubsystem,
			Name:      "health",
			Help:      "Current inner-ring node state.",
		})
	)

	mustRegister(epoch)
	mustRegister(health)

	return &InnerRingServiceMetrics{
		epoch:  epoch,
		health: health,
	}
}

// SetEpoch updates epoch metrics.
func (m InnerRingServiceMetrics) SetEpoch(epoch uint64) {
	m.epoch.value.Set(float64(epoch))
}

// SetHealth updates health metrics.
func (m InnerRingServiceMetrics) SetHealth(s int32) {
	m.health.value.Set(float64(s))
}
