package metrics

import "github.com/prometheus/client_golang/prometheus"

const namespace = "frostfs_node"

type NodeMetrics struct {
	objectServiceMetrics
	engineMetrics
	stateMetrics
	replicatorMetrics
	epoch metric[prometheus.Gauge]

	writeCacheMetrics *writeCacheMetrics
}

func NewNodeMetrics() *NodeMetrics {
	objectService := newObjectServiceMetrics()
	objectService.register()

	engine := newEngineMetrics()
	engine.register()

	state := newStateMetrics()
	state.register()

	replicator := newReplicatorMetrics()
	replicator.register()

	epoch := newGauge(prometheus.GaugeOpts{
		Namespace: namespace,
		Subsystem: innerRingSubsystem,
		Name:      "epoch",
		Help:      "Current epoch as seen by inner-ring node.",
	})
	mustRegister(epoch)

	writeCacheMetrics := newWriteCacheMetrics()
	writeCacheMetrics.register()

	return &NodeMetrics{
		objectServiceMetrics: objectService,
		engineMetrics:        engine,
		stateMetrics:         state,
		replicatorMetrics:    replicator,
		epoch:                epoch,
		writeCacheMetrics:    writeCacheMetrics,
	}
}

// SetEpoch updates epoch metric.
func (m *NodeMetrics) SetEpoch(epoch uint64) {
	m.epoch.value.Set(float64(epoch))
}

// WriteCache returns WriteCache metrics.
func (m *NodeMetrics) WriteCache() WriteCacheMetrics {
	if m == nil {
		return nil
	}
	return m.writeCacheMetrics
}
